package geniusweb.actions;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonTypeName;

import geniusweb.issuevalue.Bid;

/**
 * A vote is an indication by a party that it conditionally accepts a bid. It's
 * then up to the protocol to determine the outcome.
 */
@JsonTypeName("vote")
public class Vote extends ActionWithBid {
	private final Integer minPower;

	/**
	 * @param id       the {@link PartyId} that does the action
	 * @param bid      the bid that is voted on
	 * @param minPower the minimum power this bid must get in order for the vote
	 *                 to be valid. If power=1 for all participants (usually the
	 *                 default) power can be interpreted as number of votes
	 */
	@JsonCreator
	public Vote(@JsonProperty("actor") PartyId id, @JsonProperty("bid") Bid bid,
			@JsonProperty("minPower") Integer minPower) {
		super(id, bid);
		this.minPower = minPower;
		if (bid == null || minPower == null || minPower < 1) {
			throw new IllegalArgumentException(
					"Vote must have non-null bid and minVotes, and minVotes must be >=1");
		}
	}

	/**
	 * 
	 * @return the minimum number of votes this bid must get in order for the
	 *         vote to be valid.
	 */
	public Integer getMinPower() {
		return minPower;
	}

	@Override
	public String toString() {
		return "Vote[" + getActor() + "," + getBid() + "," + minPower + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result
				+ ((minPower == null) ? 0 : minPower.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		Vote other = (Vote) obj;
		if (minPower == null) {
			if (other.minPower != null)
				return false;
		} else if (!minPower.equals(other.minPower))
			return false;
		return true;
	}

}
